use itertools::Itertools;
use std::collections::HashSet;

fn main() {
    let input = std::fs::read_to_string("input/day3.txt").unwrap();
    println!("part 1: {}", part1(&input));
    println!("part 1: {}", part2(&input));
}

fn part1(input: &str) -> i64 {
    input
        .lines()
        .map(|line| line.split_at(line.len() / 2))
        .flat_map(|(first, second)| {
            HashSet::<_>::intersection(&first.chars().collect(), &second.chars().collect())
                .map(|c| *c as u8)
                .collect::<Vec<_>>()
        })
        .map(|item| match item {
            b'a'..=b'z' => item - b'a' + 1,
            b'A'..=b'Z' => item - b'A' + 27,
            _ => panic!("input was wrong"),
        } as i64)
        .sum()
}

fn part2(input: &str) -> i64 {
    input
        .lines()
        .chunks(3)
        .into_iter()
        .filter_map(|group| {
            group
                .map(|line| line.chars().collect::<HashSet<_>>())
                .reduce(|previous, badges| previous.intersection(&badges).copied().collect())
        })
        .flatten()
        .map(|item| match item {
            'a'..='z' => item as u8 - b'a' + 1,
            'A'..='Z' => item as u8 - b'A' + 27,
            _ => panic!("input was wrong"),
        } as i64)
        .sum()
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn examples() {
        let example = "vJrwpWtwJgWrhcsFMMfFFhFp\njqHRNqRjqzjGDLGLrsFMfFZSrLrFZsSL\nPmmdzqPrVvPwwTWBwg\nwMqvLMZHhHMvwLHjbvcjnnSBnvTQFn\nttgJtRGJQctTZtZT\nCrZsJsPPZsGzwwsLwLmpwMDw";
        assert_eq!(part1(example), 157);
        assert_eq!(part2(example), 70);
    }
}
